Docker Platform Cookbook
========================

Description
-----------

[Docker][] is an open-source project that automates the deployment of
applications inside software containers, by providing an additional layer of
abstraction and automation of operating-system-level virtualisation on Linux.
Docker uses the resource isolation features of the Linux kernel such as cgroups
and kernel namespaces, and a union-capable file system such as aufs and others
to allow independent "containers" to run within a single Linux instance,
avoiding the overhead of starting and maintaining virtual machines.

This cookbook provides recipes to install and configure Docker with
configuration stored in attributes. It uses [yum-docker cookbook][] to install
official repository and it wraps [docker cookbook][] resources to control
Docker status.

Requirements
------------

### Cookbooks and gems

Declared in [metadata.rb](metadata.rb) and in [Gemfile](Gemfile).

### Platforms

- RHEL Family 7, tested on Centos

Note: by composing it with [apt-docker cookbook][], it should work fine on
Debian / Ubuntu.

Usage
-----

### Test

This cookbook is fully tested in test-kitchen, using the vagrant driver and
inspec verifier.

For more information, see [.kitchen.yml](.kitchen.yml) and [test](test)
directory.

Attributes
----------

Configuration is done by overriding default attributes. All configuration keys
have a default defined in [attributes/default.rb](attributes/default.rb).
Please read it to have a comprehensive view of what and how you can configure
this cookbook behavior.

In addition, you can use attributes to define [docker cookbook][] resources.
All you have to do is to define an attribute of the same name as the resource
you want to call containing the different instances of this resource. For
instance:

```json
"docker_volume": {
  "data": {
    "action": "create"
  },
  "to_remove": {
    "action": "remove"
  }
}
```

It creates a docker volume named `data` and remove the one called `to_remove`.
The key of each sub hash table is the name attribute and the value is an hash
table containing the resource configuration, mapping exactly the attribute name
defined in [docker cookbook][] documentation.

If you need multiple arguments for an attribute, you have to declare them in
an array. If you need an array as single argument, add an extra array.

For instance:

```json
"docker_container": {
  "my_alpine": {
    "container_name": "alpine",
    "repo": "alpine",
    "volumes": [["data:/data"]],
    "action": "run",
    "subscribes": ["redeploy", "docker_image[alpine]", "immediately"]
  }
}
```

You can find other examples in [.kitchen.yml](.kitchen.yml) file.

Recipes
-------

### default

Include `repository`, `package`, `service` and `docker` recipes.

### repository

Use [yum-docker cookbook][] to install official yum repository.

### package

Install `docker-engine` package or upgrade it if attribute version is set to
`latest`.

### service

Enable and start `docker` service. Restart it if package was upgraded.

### docker

Wrap all [docker cookbook][] resources so it is possible to use them with
attributes. Read [attributes](#attributes) section for more details.

Resources/Providers
-------------------

None.

Changelog
---------

Available in [CHANGELOG.md](CHANGELOG.md).

Contributing
------------

Please read carefully [CONTRIBUTING.md](CONTRIBUTING.md) before making a merge
request.

License and Author
------------------

- Author:: Samuel Bernard (<samuel.bernard@s4m.io>)

```text
Copyright (c) 2016 Sam4Mobile

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
```

[docker]: https://docker.io
[apt-docker cookbook]: https://supermarket.chef.io/cookbooks/apt-docker
[yum-docker cookbook]: https://supermarket.chef.io/cookbooks/yum-docker
[docker cookbook]: https://supermarket.chef.io/cookbooks/docker
